'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const node_child_process = require('node:child_process');
const path = require('node:path');
const vite = require('vite');
const makeDebugger = require('debug');
const c = require('picocolors');
const utils = require('@antfu/utils');
const minimatch = require('minimatch');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const path__default = /*#__PURE__*/_interopDefaultLegacy(path);
const makeDebugger__default = /*#__PURE__*/_interopDefaultLegacy(makeDebugger);
const c__default = /*#__PURE__*/_interopDefaultLegacy(c);

const PLUGIN_NAME = "vite:plugin:run";
const debug = makeDebugger__default(PLUGIN_NAME);
const throttles = /* @__PURE__ */ new Set();
function warn(prefix, message) {
  process.stdout.write(c__default.bold(`${c__default.yellow(`(!) ${prefix}`)} ${message}
`));
}
function run(options = []) {
  const silent = "silent" in options ? options.silent : void 0;
  const skipDts = "skipDts" in options ? options.skipDts : void 0;
  const resolvedOptions = {
    env: {},
    silent: silent ?? true,
    skipDts: skipDts ?? true,
    input: utils.toArray("input" in options ? options.input : options)
  };
  return {
    name: PLUGIN_NAME,
    configResolved(config) {
      resolvedOptions.env = vite.loadEnv(config.mode ?? process.env.NODE_ENV ?? "development", process.cwd(), "");
      debug("Given options:", options);
      debug("Resolved options:", resolvedOptions);
      resolvedOptions.input.forEach((runner) => {
        if (runner.startup !== false || runner.condition === void 0) {
          handleRunnerCommand(resolvedOptions, runner);
        }
      });
    },
    handleHotUpdate({ file, server }) {
      if (resolvedOptions.skipDts && file.endsWith(".d.ts")) {
        return [];
      }
      handleReload(resolvedOptions, { file, server });
    }
  };
}
function handleReload(options, parameters) {
  const file = parameters.file.replaceAll("\\", "/");
  options.input.forEach((runner) => {
    const name = getRunnerName(runner);
    if (runner.condition && !runner.condition(file)) {
      debug(`${name} condition did not pass for ${c__default.gray(parameters.file)}`);
      return;
    }
    const patterns = !Array.isArray(runner.pattern) ? [runner.pattern].filter(Boolean) : runner.pattern;
    const patternMatch = patterns.some((pattern) => {
      pattern = path__default.resolve(parameters.server.config.root, pattern);
      if (minimatch.minimatch(file, pattern)) {
        debug(`${name} pattern ${pattern} matched for ${c__default.gray(parameters.file)}`);
        return true;
      }
      return false;
    });
    if (!patternMatch) {
      debug(`${name} no pattern matched for ${c__default.gray(parameters.file)} (${patterns.map((p) => path__default.resolve(parameters.server.config.root, p))})`);
      return;
    }
    handleRunner(runner, options, parameters);
  });
}
function handleRunner(runner, options, parameters) {
  debug(`${c__default.gray(parameters.file)} changed, applying its handler...`);
  try {
    if (typeof runner.onFileChanged === "function") {
      runner.onFileChanged?.(parameters);
    }
    if (Array.isArray(runner.run)) {
      handleRunnerCommand(options, runner);
    }
  } catch (error) {
    warn(PLUGIN_NAME, `Handler failed for ${parameters.file}: ${error.message}`);
    debug("Full error:", error);
  }
}
function handleRunnerCommand(options, runner) {
  if (!runner.run) {
    return;
  }
  const name = getRunnerName(runner);
  if (throttles.has(name)) {
    debug(`${name} is throttled.`);
    return;
  }
  throttles.add(name);
  setTimeout(() => throttles.delete(name), runner.throttle ?? 500);
  debug(`Running ${name}...`);
  setTimeout(() => {
    const child = node_child_process.spawn(getExecutable(options, getRunnerCommand(runner)), getRunnerArguments(runner), { shell: true });
    if (!options.silent) {
      child.stdout.on("data", (data) => {
        process.stdout.write(data.toString());
      });
      child.stderr.on("data", (data) => {
        process.stdout.write(data.toString());
      });
    }
    child.on("close", (code) => {
      const result = code === 0 ? "ran successfully" : `failed with code ${code}`;
      debug(`${name} ${result}`);
    });
  }, runner.delay ?? 50);
}
function getRunnerName(runner) {
  return c__default.bold(`[${`${runner.name}` ?? getRunnerCommand(runner) ?? "<runner>"}]`);
}
function getRunnerArguments(runner) {
  const args = typeof runner.run === "function" ? runner.run() : runner.run ?? [];
  return args.slice(1) ?? [];
}
function getRunnerCommand(runner) {
  const args = typeof runner.run === "function" ? runner.run() : runner.run ?? [];
  return args?.[0];
}
function getExecutable(options, name) {
  if (!name) {
    throw new Error("No executable given.");
  }
  return process.env[`${name}_PATH`] || options.env[`${name}_PATH`] || name;
}

exports["default"] = run;
exports.run = run;
